'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function () {
  return {
    visitor: {
      Program: function Program(programPath) {
        // On program start, do an explicit traversal up front for this plugin.
        programPath.traverse({
          ObjectProperty: {
            exit: function exit(path) {
              var node = path.node;

              if (node.computed || node.key.name !== 'propTypes') {
                return;
              }

              var parent = path.findParent(function (currentNode) {
                if (currentNode.type !== 'CallExpression') {
                  return false;
                }

                return currentNode.get('callee').matchesPattern('React.createClass');
              });

              if (parent) {
                path.remove();
              }
            }
          },
          ClassProperty: function ClassProperty(path) {
            var node = path.node;
            var scope = path.scope;


            if (node.key.name === 'propTypes') {
              var superClass = scope.path.get('superClass');

              if (isReactClass(superClass, scope)) {
                path.remove();
              }
            }
          },
          AssignmentExpression: function AssignmentExpression(path) {
            var node = path.node;
            var scope = path.scope;


            if (node.left.computed || !node.left.property || node.left.property.name !== 'propTypes') {
              return;
            }

            var className = node.left.object.name;
            var binding = scope.getBinding(className);

            if (!binding) {
              return;
            }

            if (binding.path.isClassDeclaration()) {
              var superClass = binding.path.get('superClass');

              if (isReactClass(superClass, scope)) {
                path.remove();
              }
            } else if ((0, _isStatelessComponent2.default)(binding.path)) {
              path.remove();
            }
          }
        });
      }
    }
  };
};

var _isStatelessComponent = require('./isStatelessComponent');

var _isStatelessComponent2 = _interopRequireDefault(_isStatelessComponent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isReactClass(superClass, scope) {
  var answer = false;

  if (superClass.matchesPattern('React.Component') || superClass.node.name === 'Component') {
    answer = true;
  } else if (superClass.node.name) {
    // Check for inheritance
    var className = superClass.node.name;
    var binding = scope.getBinding(className);
    superClass = binding.path.get('superClass');

    // superClass.node === undefined for BaseComponent and PureComponent so
    // adding a hacky check here
    if (className === 'BaseComponent' || className === 'PureComponent') {
      answer = true;
    } else if (superClass.matchesPattern('React.Component') || superClass.node && superClass.node.name === 'Component') {
      answer = true;
    }
  }

  return answer;
}